// license:BSD-3-Clause
// copyright-holders:Olivier Galibert
#ifndef MAME_FORMATS_THOM_DSK_H
#define MAME_FORMATS_THOM_DSK_H

#pragma once

#include "wd177x_dsk.h"

class thomson_format : public wd177x_format
{
public:
  thomson_format();

  virtual const char *name() const noexcept override;
  virtual const char *description() const noexcept override;
  virtual const char *extensions() const noexcept override;
  virtual bool load(util::random_read &io, uint32_t form_factor, const std::vector<uint32_t> &variants, floppy_image &image) const override;

private:
  static const format formats[];

protected:
  thomson_format(const format *formats);
  bool validate_fat(util::random_read &io, const format &f, int offset) const;
  bool validate_catalog(util::random_read &io, const format &f, int offset) const;
  int find_struct(util::random_read &io, uint32_t form_factor, const std::vector<uint32_t> &variants) const;
  virtual int find_size(util::random_read &io, uint32_t form_factor, const std::vector<uint32_t> &variants) const override;
  virtual int identify(util::random_read &io, uint32_t form_factor, const std::vector<uint32_t> &variants) const override;
  virtual int get_image_offset(const format &f, int head, int track) const override;
  virtual const wd177x_format::format &get_track_format(const format &f, int head, int track) const override;
  virtual floppy_image_format_t::desc_e* get_desc_fm(const format &f, int &current_size, int &end_gap_index) const override;
  virtual floppy_image_format_t::desc_e* get_desc_mfm(const format &f, int &current_size, int &end_gap_index) const override;
};

extern const thomson_format FLOPPY_THOMSON_FORMAT;

#endif // MAME_FORMATS_THOM_DSK_H
